from typing import NamedTuple

from rich.syntax import PygmentsSyntaxTheme
from textual.color import BLACK, WHITE, Color


class WidgetColors(NamedTuple):
    contrast_text_color: Color
    bgcolor: Color
    selection_bgcolor: Color

    @classmethod
    def from_theme(cls, theme: str) -> "WidgetColors":
        theme_background_style = PygmentsSyntaxTheme(theme).get_background_style()
        if (
            theme_background_style is not None
            and theme_background_style.bgcolor is not None
        ):
            t_color = Color.from_rich_color(theme_background_style.bgcolor)
            bgcolor = t_color
            contrast_text_color = t_color.get_contrast_text()
            if t_color.brightness >= 0.5:
                selection_bgcolor = t_color.darken(0.10)
            else:
                selection_bgcolor = t_color.lighten(0.10)
            return WidgetColors(contrast_text_color, bgcolor, selection_bgcolor)
        else:
            return WidgetColors(BLACK, WHITE, Color.parse("#aaaaaa"))



from typing import Union

from textual.containers import Container, ScrollableContainer
from textual.widget import Widget

from textual_textarea.colors import WidgetColors


class TextContainer(
    ScrollableContainer,
    inherit_bindings=False,
    can_focus=False,
    can_focus_children=True,
):
    DEFAULT_CSS = """
        TextContainer {
            height: 1fr;
            width: 100%
        }
    """


class FooterContainer(
    Container,
    inherit_bindings=False,
    can_focus=False,
    can_focus_children=True,
):
    DEFAULT_CSS = """
        FooterContainer {
            dock: bottom;
            height: auto;
            width: 100%
        }
    """

    def __init__(
        self,
        theme_colors: WidgetColors,
        *children: Widget,
        name: Union[str, None] = None,
        id: Union[str, None] = None,
        classes: Union[str, None] = None,
        disabled: bool = False,
    ) -> None:
        super().__init__(
            *children, name=name, id=id, classes=classes, disabled=disabled
        )
        self.theme_colors = theme_colors



import re
from typing import List, NamedTuple

from textual import log
from textual.events import MouseEvent

WWB = re.compile(r"\W*\w+\b")


class Cursor(NamedTuple):
    lno: int
    pos: int

    @classmethod
    def from_mouse_event(cls, event: MouseEvent) -> "Cursor":
        return Cursor(event.y, event.x - 1)


def handle_arrow(key: str, lines: List[str], cursor: Cursor) -> Cursor:
    arrow = key.split("+")[-1]
    if "ctrl" in key:
        assert arrow not in ("up", "down"), "ctrl+up/down should be handled first"
        if arrow == "right":
            return _handle_ctrl_right(lines, cursor)
        else:  # if arrow == "left":
            return _handle_ctrl_left(lines, cursor)
    else:
        if arrow == "right":
            return _handle_right(lines, cursor)
        elif arrow == "left":
            return _handle_left(lines, cursor)
        elif arrow == "down":
            return _handle_down(lines, cursor)
        else:  # arrow == "up":
            return _handle_up(lines, cursor)


def _handle_right(lines: List[str], cursor: Cursor) -> Cursor:
    max_x = len(lines[cursor.lno]) - 1
    max_y = len(lines) - 1
    if cursor.lno == max_y:
        return Cursor(lno=max_y, pos=min(max_x, cursor.pos + 1))
    elif cursor.pos == max_x:
        return Cursor(lno=cursor.lno + 1, pos=0)
    else:
        return Cursor(lno=cursor.lno, pos=cursor.pos + 1)


def _handle_left(lines: List[str], cursor: Cursor) -> Cursor:
    if cursor.lno == 0:
        return Cursor(0, pos=max(0, cursor.pos - 1))
    elif cursor.pos == 0:
        return Cursor(
            lno=cursor.lno - 1,
            pos=len(lines[cursor.lno - 1]) - 1,
        )
    else:
        return Cursor(lno=cursor.lno, pos=cursor.pos - 1)


def _handle_down(lines: List[str], cursor: Cursor) -> Cursor:
    max_y = len(lines) - 1
    if cursor.lno == max_y:
        return Cursor(lno=max_y, pos=len(lines[cursor.lno]) - 1)
    else:
        max_x = len(lines[cursor.lno + 1]) - 1
        return Cursor(lno=cursor.lno + 1, pos=min(max_x, cursor.pos))


def _handle_up(lines: List[str], cursor: Cursor) -> Cursor:
    if cursor.lno == 0:
        return Cursor(0, 0)
    else:
        max_x = len(lines[cursor.lno - 1]) - 1
        return Cursor(lno=cursor.lno - 1, pos=min(max_x, cursor.pos))


def _handle_ctrl_right(lines: List[str], cursor: Cursor) -> Cursor:
    max_x = len(lines[cursor.lno]) - 1
    max_y = len(lines) - 1
    if cursor.pos == max_x and cursor.lno == max_y:
        return cursor
    elif cursor.pos == max_x:
        lno = cursor.lno + 1
        pos = 0
    else:
        lno = cursor.lno
        pos = cursor.pos

    tail = lines[lno][pos:]
    if match := WWB.match(tail):
        return Cursor(lno=lno, pos=pos + match.span()[1])
    else:  # no more words, move to end of line
        return Cursor(lno=lno, pos=len(lines[lno]) - 1)


def _handle_ctrl_left(lines: List[str], cursor: Cursor) -> Cursor:
    if cursor.pos == 0 and cursor.lno == 0:
        return cursor
    elif cursor.pos == 0:
        lno = cursor.lno - 1
        pos = len(lines[lno]) - 1
    else:
        lno = cursor.lno
        pos = cursor.pos

    tail = lines[lno][:pos][::-1]
    if match := WWB.match(tail):
        log(match)
        return Cursor(lno=lno, pos=pos - match.span()[1])
    else:  # no more words, move to start of line
        return Cursor(lno=lno, pos=0)



import sys

from textual.app import App, ComposeResult

from textual_textarea import TextArea


class TextApp(App, inherit_bindings=False):
    def compose(self) -> ComposeResult:
        try:
            language = sys.argv[1]
        except IndexError:
            language = "python"
        yield TextArea(language=language, theme="monokai", use_system_clipboard=True)

    def on_mount(self) -> None:
        ta = self.query_one(TextArea)
        ta.focus()


app = TextApp()
app.run()



from typing import List


def serialize_lines(lines: List[str]) -> str:
    return "\n".join([line.rstrip() for line in lines])


def deserialize_lines(text: str, trim: bool = False) -> List[str]:
    if text:
        lines = [f"{line} " for line in text.splitlines()]
        if text.endswith(("\n", "\r", "\r\n")):
            lines.append(" ")
    else:
        lines = [" "]

    if trim:
        lines[-1] = lines[-1].rstrip()

    return lines
